@@ -12,6 +12,8 @@ use bytes::Buf;
 use bytes::Bytes;
 use rss::Channel;
 use rss::Item;
+use smartcow::SmartCow;
+use smartstring::alias::String;
 
 mod error;
 pub use error::Error;
@@ -21,8 +23,8 @@ pub use torrent::Torrent;
 #[derive(Clone)]
 pub struct Client {
 	http: reqwest::Client,
-	base_url: String,
-	apikey: String
+	base_url: std::string::String,
+	apikey: std::string::String
 }
 
 impl Client {
@@ -38,53 +40,53 @@ impl Client {
 		Ok(this)
 	}
 
-	async fn get(&self, t: &str, mut qparams: BTreeMap<String, String>) -> Result<Bytes, reqwest::Error> {
-		qparams.insert("apikey".to_string(), self.apikey.clone());
+	async fn get(&self, t: &str, mut qparams: BTreeMap<&str, SmartCow<'_>>) -> Result<Bytes, reqwest::Error> {
+		qparams.insert("apikey", SmartCow::Borrowed(&self.apikey));
 		let url = format!("{}?t={}&{}", self.base_url, &t, qparams.into_iter()
-			.map(|(k, v)| format!("{}={}", urlencoding::encode(&k), urlencoding::encode(&v)))
+			.map(|(k, v)| format!("{}={}", urlencoding::encode(&k), urlencoding::encode(&v)).into())
 			.collect::<Vec<String>>()
 			.join("&")
 		);
 		Ok(self.http.get(&url).send().await?.error_for_status()?.bytes().await?)
 	}
 
-	async fn get_items(&self, t: &str, qparams: BTreeMap<String, String>) -> Result<Vec<Item>, Error> {
+	async fn get_items(&self, t: &str, qparams: BTreeMap<&str, SmartCow<'_>>) -> Result<Vec<Item>, Error> {
 		let bytes = self.get(t, qparams).await?;
 		let channel = tokio::task::spawn_blocking(move || Channel::read_from(bytes.chunk())).await??;
 		Ok(channel.into_items())
 	}
 
 	pub async fn tvsearch(&self, q: Option<&str>, rid: Option<u32>, tvdbid: Option<u32>, tvmazeid: Option<u32>, season: Option<u16>, ep: Option<u16>) -> Result<Vec<Result<Torrent, Error>>, Error> {
-		let mut qparams = BTreeMap::<String, String>::new();
+		let mut qparams = BTreeMap::<&str, SmartCow>::new();
 		if let Some(v) = q {
-			qparams.insert("q".to_string(), v.to_string());
+			qparams.insert("q", SmartCow::Borrowed(v));
 		}
 		if let Some(v) = rid {
-			qparams.insert("rid".to_string(), v.to_string());
+			qparams.insert("rid", SmartCow::Owned(String::from(v.to_string())));
 		}
 		if let Some(v) = tvdbid {
-			qparams.insert("tvdbid".to_string(), v.to_string());
+			qparams.insert("tvdbid", SmartCow::Owned(String::from(v.to_string())));
 		}
 		if let Some(v) = tvmazeid {
-			qparams.insert("tvmazeid".to_string(), v.to_string());
+			qparams.insert("tvmazeid", SmartCow::Owned(String::from(v.to_string())));
 		}
 		if let Some(v) = season {
-			qparams.insert("season".to_string(), v.to_string());
+			qparams.insert("season", SmartCow::Owned(String::from(v.to_string())));
 		}
 		if let Some(v) = ep {
-			qparams.insert("ep".to_string(), v.to_string());
+			qparams.insert("ep", SmartCow::Owned(String::from(v.to_string())));
 		}
 		let items = self.get_items("tvsearch", qparams).await?;
 		Ok(items.into_iter().map(Torrent::try_from).collect::<Vec<_>>())
 	}
 
 	pub async fn moviesearch(&self, q: Option<&str>, imdbid: Option<u64>) -> Result<Vec<Result<Torrent, Error>>, Error> {
-		let mut qparams = BTreeMap::<String, String>::new();
+		let mut qparams = BTreeMap::<&str, SmartCow>::new();
 		if let Some(v) = q {
-			qparams.insert("q".to_string(), v.to_string());
+			qparams.insert("q", SmartCow::Borrowed(v));
 		}
 		if let Some(v) = imdbid {
-			qparams.insert("imdbid".to_string(), v.to_string());
+			qparams.insert("imdbid", SmartCow::Owned(String::from(v.to_string())));
 		}
 		let items = self.get_items("movies", qparams).await?;
 		Ok(items.into_iter().map(Torrent::try_from).collect::<Vec<_>>())
